/************************************************************************
File Name:	Invsqrt.asm

Date Modified:	05/22/00		GJO
				05/25/00		GJO	Code verified on rev 0.1
				09/19/04		JDT Ported to VDSP++ 3.5. replace .SECTION syntax

Purpose:	ADSP-21364 multichannel SIMD subroutine to compute the 
			inverse square root of two samples, y1 and y2, to 
			single-precision (24 bits mantissa) using the 1/sqrt(y)
		 	approximation.
	
			Equation:	X = invsqrt(Y) = 1/sqrt(Y)

Calling Parameters:
			b0 = address location of input
			b1 = address location of result
			l0, l1=0 setup modulo addressing
			m1 = setup address increment modifier

Assumptions:
			The start of the data set is indexed by i0.
Return Values:
		    F0 = isqrtsgl(Y)

Registers Affected:
			i0, i1, F0, F1, F4, F8, F12

Cycle Count: 16
			2 cycles for SIMD initialization
			3 cycles for setup 
			9 cycles for invsqrt()
			2 cycles for output and disabling SIMD

************************************************************************/

#include	"def21364.h"

.GLOBAL isqrtsgl;

/* Program memory code */
.SECTION/PM     seg_pmco;

isqrtsgl:     	bit set mode1 PEYEN ; 			/* Enable SIMD mode */
				nop;
				F1 = 0.5;
				F8 = 3.0;
				F0 = dm(i0,m1); 	/* Load PEx/PEy registers with input data */
				F4=RSQRTS F0;       /* Fetch seed */
                F12=F4*F4;          /* F12=X0^2 */
                F12=F12*F0;         /* F12=C*X0^2 */
                F4=F1*F4, F12=F8-F12;           /* F4=.5*X0, F10=3-C*X0^2 */ 
                F4=F4*F12;          /* F4=X1=.5*X0(3-C*X0^2) */
                F12=F4*F4;          /* F12=X1^2 */
                F12=F12*F0;			/* F12=C*X1^2 */
                F4=F1*F4, F12=F8-F12;  			/* F4=.5*X1, F10=3-C*X1^2 */
                F0=F4*F12;          /* F4=X2=.5*X1(3-C*X1^2) */
				RTS(DB); 
				dm(i1,m1)=F0;			/* Store output */
				bit clr mode1 PEYEN ; 	/* Disable SIMD mode */


